/*------------------------------------------------------------------------------------------*\
 *   Copyright (C) 2011,2012 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
\*------------------------------------------------------------------------------------------*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <asm/byteorder.h>
#include <linux/init.h>
#include <linux/errno.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/workqueue.h>
#include <linux/sched.h>
#include <linux/spinlock.h>
#include <linux/ethtool.h>

#include <asm/irq.h>
#include <asm/io.h>
#include <asm/prom.h>
#include <net/sch_generic.h>
#include <asm/mach_avm.h>

#include <linux/if_vlan.h>

#include "avmnet_debug.h"
#include "avmnet_module.h"
#include "avmnet_config.h"
#include "avmnet_common.h"
#include "../management/avmnet_links.h"

#include <atheros.h>
#include "ag934x.h"
#include "atheros_mac.h"

static struct resource ath_gmac_gpio = {
    .flags = IORESOURCE_IO,
    .start = -1,
    .end   = -1
};

static unsigned athr_tx_desc_num[ATHR_GMAC_NMACS] = { CONFIG_ATHR_GMAC_NUMBER_TX_PKTS, CONFIG_ATHR_GMAC_NUMBER_TX_PKTS_1 };
static unsigned athr_rx_desc_num[ATHR_GMAC_NMACS] = { CONFIG_ATHR_GMAC_NUMBER_RX_PKTS, CONFIG_ATHR_GMAC_NUMBER_RX_PKTS_1 };


#ifdef ATHR_GMAC_SUPPORT_MULTIPLE_DS
static int tx_len_per_ds = 0;
#endif
static int tx_max_desc_per_ds_pkt = 2;      /*--- darf im nicht kleiner als 2 sein, sonst funktioniert athr_gmac_tx_reap nicht ---*/
int frame_sz = 0;
extern int fifo_3;

/*------------------------------------------------------------------------------------------*\
 * allocate and init rings, descriptors etc.
\*------------------------------------------------------------------------------------------*/
static inline void athr_gmac_desc_alloc(athr_gmac_ring_t *r, int count) {

    r->ring_desc  =  (athr_gmac_desc_t *)dma_alloc_coherent(NULL,
                                                            sizeof(athr_gmac_buffer_t) * count, 
                                                            &r->ring_desc_dma, 
                                                            GFP_DMA);
    memset(r->ring_desc, 0, sizeof(athr_gmac_buffer_t) * count);

    AVMNET_INFO("[%s] ring_desc 0x%p ring_desc_dma 0x%p len 0x%x\n", __func__, r->ring_desc, (void *) r->ring_desc_dma, sizeof(athr_gmac_buffer_t) * count);

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if 0
void athr_print_descriptors(struct athmac_context *mac) {
    unsigned int i;
    athr_gmac_ring_t *r;
    athr_gmac_desc_t *dc;
    r = &mac->mac_rxring; 
    dc = r->ring_desc;
    printk("[%s] head %d tail %d anzahl %d\n", __FUNCTION__, 
            r->ring_head, 
            r->ring_tail, 
            r->ring_nelem);
    for(i = 0 ; i < r->ring_nelem; i++, dc++ ) {
        printk("[%3d] %#p -> %#p %08x %c %c %d\n", 
                i,
                dc,
                dc->next_desc,
                dc->pkt_start_addr,
                dc->is_empty ? 'E' : '-',
                dc->more ? 'M' : '-',
                dc->pkt_size
              );
    }
}
#endif

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int athr_gmac_ring_alloc(athr_gmac_ring_t *r, int count) {
    int buf_alloc_size;

    buf_alloc_size  = sizeof(athr_gmac_buffer_t) * count;

    memset(r, 0, sizeof(athr_gmac_ring_t));

    r->ring_buffer = (athr_gmac_buffer_t *)kmalloc(buf_alloc_size, GFP_KERNEL);
    memset(r->ring_buffer, 0, buf_alloc_size);

    r->mac_txheader = (athr_mac_packet_header_t *)kmalloc(sizeof(athr_mac_packet_header_t) * count, GFP_KERNEL);

    AVMNET_DEBUG("[%s] Allocated %d at 0x%lx\n", __func__, buf_alloc_size, (unsigned long)r->ring_buffer);

    if (!r->ring_buffer) {
        AVMNET_ERR("[%s]: unable to allocate buffers\n", __func__);
        panic("[%s]: unable to allocate buffers\n", __func__);
    }

    athr_gmac_desc_alloc(r, count * 2);

    if (! r->ring_desc) {
        panic("[%s] : unable to allocate coherent descs\n", __func__);
    }

    r->ring_nelem   = count;

    return 0;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_buffer_free(struct sk_buff *skb) {

    AVMNET_DEBUG("[%s] skb 0x%p skb->data 0x%p\n", __func__, skb, skb->data);
    if (in_irq())
        dev_kfree_skb_irq(skb);
    else
        dev_kfree_skb_any(skb);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_ring_release(athr_gmac_ring_t  *r) {
    int i;

    for(i = 0; i < r->ring_nelem; i++) {
        if (r->ring_buffer[i].buf_pkt) {
            dev_kfree_skb_any(r->ring_buffer[i].buf_pkt);
        }
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_ring_free(athr_gmac_ring_t *r) {

    dma_free_coherent(NULL, sizeof(athr_gmac_desc_t)*r->ring_nelem, r->ring_desc, r->ring_desc_dma);
    kfree(r->ring_buffer);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_rx_free(struct athmac_context *mac) {
    athr_gmac_ring_release(&mac->mac_rxring);
    athr_gmac_ring_free(&mac->mac_rxring);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_tx_free(struct athmac_context *mac) {
    unsigned int ac;
    
    for(ac = 0;ac < mac->mac_noacs; ac++) {
        athr_gmac_ring_release(&mac->mac_txring[ac]);
        athr_gmac_ring_free(&mac->mac_txring[ac]);
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int athr_gmac_tx_alloc(struct athmac_context *mac) {
    athr_gmac_ring_t *r ;
    athr_gmac_desc_t *ds;
    unsigned int ac;
    int ringlen;
    int i, next;

    AVMNET_DEBUG("[%s] mac->mac_noacs %d mac->num_tx_desc %d\n", __func__, mac->mac_noacs, mac->num_tx_desc);
    for(ac = 0; ac < mac->mac_noacs; ac++) {

        r  = &mac->mac_txring[ac];
        if (athr_gmac_ring_alloc(r, mac->num_tx_desc))
            return 1;

        AVMNET_DEBUG("[%s] r->ring_nelem %d\n", __func__, r->ring_nelem);

        /*----------------------------------------------------------------------------------*\
         * Es werden immer descriptor paare allozeiert, deshalb sind in Wirklichkeit
         * ring_nelem * 2 Descriptoren vorhanden
        \*----------------------------------------------------------------------------------*/
        ds = r->ring_desc;
        ringlen = r->ring_nelem * 2;
        for(i = 0; i < ringlen ; i++ ) {
            next             =   (i == (ringlen - 1)) ? 0 : (i + 1);
            ds[i].next_desc  =   athr_gmac_desc_dma_addr(r, &ds[next]);
            athr_gmac_tx_own(&ds[i]);
        }
    }
   return 0;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static struct sk_buff * athr_gmac_buffer_alloc(void) {

#if 0
    struct sk_buff *skb;
    unsigned long off;

    skb = dev_alloc_skb(ATHR_GMAC_RX_BUF_SIZE + frame_sz);
    if (unlikely(!skb))
        return NULL;

    off = ((unsigned long) skb->data) & (L1_CACHE_BYTES - 1);
    AVMNET_ERR("[%s] skb->data %p off %d\n", __func__, skb->data, off);

    if (off != 0)
        skb_reserve(skb, L1_CACHE_BYTES - off);

    /*--- Return unaligned address for skb data so that header processing will be aligned ---*/ 
    /*--- skb_reserve(skb, (ATHR_GMAC_RX_RESERVE + 2)); ---*/

#if 0
    printk(KERN_ERR "[%s] skb 0x%p skb->data 0x%p skb->end 0x%p skb_shinfo(skb) 0x%p = 0x%08x 0x%08x\n", 
            __func__, skb, skb->data, skb->end, 
            skb_shinfo(skb),
            *((unsigned int *)skb_shinfo(skb)),
            *((unsigned int *)skb_shinfo(skb))
            );
#endif 
    return skb; 
#else
    return dev_alloc_skb(ATHR_GMAC_RX_BUF_SIZE + frame_sz);
#endif
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
struct skb_save {
    spinlock_t          spinlock;
    atomic_t            count;
    struct sk_buff_head list;
    unsigned int        reused;
    unsigned int        free;
};

static struct skb_save skb_store;
void init_skb_store(void) {

    spin_lock_init(&skb_store.spinlock);
    skb_queue_head_init(&skb_store.list);    
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_store_skb(struct sk_buff *skb) {

    unsigned long flags;

    AVMNET_DEBUG("[%s] count %d skblen %d min %d\n", __func__, atomic_read(&skb_store.count), skb_end_pointer(skb) - skb->head, (ATHR_GMAC_RX_BUF_SIZE + frame_sz));
    if ((skb_end_pointer(skb) - skb->head) >= (ATHR_GMAC_RX_BUF_SIZE + frame_sz)) {

        if (atomic_read(&skb_store.count) < 100) { 

            if (likely(atomic_read(&skb->users) == 1))
                smp_rmb();
            else if ( ! likely(atomic_dec_and_test(&skb->users))) {
                goto athr_store_skb_free;
            }
            atomic_inc(&skb_store.count);

            /*--- AVMNET_DEBUG("[%s] skb->users %d\n", __func__, skb->users); ---*/
            spin_lock_irqsave(&skb_store.spinlock, flags);
            skb_queue_head(&skb_store.list, skb);
            spin_unlock_irqrestore(&skb_store.spinlock, flags);
            return;
        }
    }

athr_store_skb_free:
    /*--- AVMNET_DEBUG("[%s] free skb\n", __func__); ---*/
    athr_gmac_buffer_free(skb);
    return;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static struct sk_buff *athr_get_skb(void) {

    struct sk_buff *skb;
    unsigned long flags;
    unsigned int len = (ATHR_GMAC_RX_BUF_SIZE + frame_sz);

    spin_lock_irqsave(&skb_store.spinlock, flags);
    skb = skb_dequeue_tail(&skb_store.list);
    spin_unlock_irqrestore(&skb_store.spinlock, flags);

    /*--- AVMNET_DEBUG("[%s] skb %p len %d skblen %d testlen %d\n", __func__, skb, len, skb ? skb->data_len : 0, skb ? skb_end_pointer(skb) - skb->head : 0); ---*/

    if (skb) {
        atomic_dec(&skb_store.count);
        if (skb_recycle_check(skb, len)) {

#if 0
        {
            unsigned long off;
            off = ((unsigned long) skb->data) & (L1_CACHE_BYTES - 1);
            if (off != 0) {
                AVMNET_DEBUG("<reserve Offset %d>\n", off);
                skb_reserve(skb, L1_CACHE_BYTES - off);
            }
        }
#endif

            skb_store.reused++;

            /*--- AVMNET_DEBUG("[%s] return skb %p reused %d free %d\n", __func__, skb, skb_store.reused, skb_store.free); ---*/
            return skb;
        }
        skb_store.free++;
        /*--- AVMNET_DEBUG("[%s] <free skb %p>\n", __func__, skb); ---*/
        dev_kfree_skb_any(skb);
    } 
    return athr_gmac_buffer_alloc();
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int athr_gmac_rx_alloc(struct athmac_context  *mac) {

    athr_gmac_ring_t *r;
    athr_gmac_desc_t *ds;
    int i, next, tail;
    athr_gmac_buffer_t *bf;

    /*--- AVMNET_DEBUG("[%s] num_rx_desc %d\n", __func__, mac->num_rx_desc); ---*/

    r = &mac->mac_rxring; 
    if (athr_gmac_ring_alloc(r, mac->num_rx_desc))
        return 1;

    ds = r->ring_desc;
    for(i = 0; i < r->ring_nelem; i++ ) {
        next = (i == (r->ring_nelem - 1)) ? 0 : (i + 1);
        ds[i].next_desc = athr_gmac_desc_dma_addr(r, &ds[next]);
    }

    tail = r->ring_tail;
    for (i = 0; i < mac->num_rx_desc; i++) {
        bf = &r->ring_buffer[tail];
        ds = &r->ring_desc[tail];

        bf->buf_pkt = athr_gmac_buffer_alloc();
        if (!bf->buf_pkt) 
            goto error;

        /*--- athr_mac_cache_inv((unsigned long)(bf->buf_pkt->data - 2), ATHR_GMAC_RX_BUF_SIZE + frame_sz - ATHR_GMAC_RX_RESERVE); ---*/
        athr_mac_cache_inv((unsigned long)(bf->buf_pkt->data), ATHR_GMAC_RX_BUF_SIZE + frame_sz);

        ds->pkt_start_addr  = virt_to_phys(bf->buf_pkt->data);

        ds->res1          = 0;
        ds->res2          = 0;
        ds->ftpp_override = 0;
        ds->res3          = 0;
        ds->more          = 0;
        athr_gmac_rx_give_to_dma(ds);
        athr_gmac_ring_incr(tail);
    }

    return 0;
error:
    AVMNET_ERR("[%s] : unable to allocate rx\n", __func__);
    athr_gmac_rx_free(mac);
    return 1;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athmac_gmac_setup_hw(struct athmac_context *mac) {
    athr_gmac_ring_t *tx, *rx = &mac->mac_rxring;
    athr_gmac_desc_t *r0, *t0;

    /* clear the rx fifo state if any */
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS));
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_CFG1, (ATHR_GMAC_CFG1_RX_EN | ATHR_GMAC_CFG1_TX_EN));

    switch (mac->mac_unit) {
        case 0:
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG2, 
                (ATHR_GMAC_CFG2_PAD_CRC_EN | ATHR_GMAC_CFG2_LEN_CHECK | ATHR_GMAC_CFG2_IF_10_100));
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, (1 << 19));
            break;
        case 1:
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG2, 
                (ATHR_GMAC_CFG2_FDX | ATHR_GMAC_CFG2_PAD_CRC_EN | ATHR_GMAC_CFG2_LEN_CHECK | ATHR_GMAC_CFG2_IF_1000)); 
            athr_gmac_reg_rmw_clear(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, ATHR_GMAC_BYTE_PER_CLK_EN);
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, ATHR_GMAC_BYTE_PER_CLK_EN);
            break;

    }

    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_0, 0x1f00);

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,ATHR_RX_FLCTL)) 
        athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG1, ATHR_GMAC_CFG1_RX_FCTL);

    if (mac_has_flag(mac,ATHR_TX_FLCTL)) 
        athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG1, ATHR_GMAC_CFG1_TX_FCTL);
#else
    /*--- enable FLOW-Control ---*/
    athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG1, ATHR_GMAC_CFG1_RX_FCTL | ATHR_GMAC_CFG1_TX_FCTL);
#endif

    /*------------------------------------------------------------------------------------------*\
     * Disable ATHR_GMAC_CFG2_LEN_CHECK to fix the bug that the mac address is mistaken as 
     * length when enabling Atheros header 
     \*------------------------------------------------------------------------------------------*/
#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,ATHR_HEADER_ENABLED))
        athr_gmac_reg_rmw_clear(mac->BaseAddress, ATHR_GMAC_CFG2, ATHR_GMAC_CFG2_LEN_CHECK);
#endif

    switch(mac->mac_unit) {
        case 0:   /*--- GBIT ---*/
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_1, 0x10ffff);
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_2, 0x00aa0055);

            /*--- Weed out junk frames (CRC errored, short collision'ed frames etc.) ---*/
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_4, 0x3ffff);
            break;
        case 1:  /*--- 100 MBit ---*/
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_1, 0x10ffff);
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_2, 0x015500aa);

            /*--- Weed out junk frames (CRC errored, short collision'ed frames etc.) ---*/
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_4, 0x3ffff);
            break;
    }

    /*--- Drop CRC Errors, Pause Frames ,Length Error frames, Truncated Frames dribble nibble and rxdv error frames. ---*/
    AVMNET_INFO("[%s] Setting Drop CRC Errors, Pause Frames and Length Error frames \n", __func__);

#if 0
#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,ATHR_HEADER_ENABLED)) {
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, 0xe6be2);
    } 
    else if ((mac->mac_unit == 0 || mac->mac_unit == 1) && (is_ar7242() || is_s16() || is_s17() || mac_has_flag(mac,ATHR_JUMBO_FR))) {
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, 0xe6be2);
    }
    if (is_f2e()) {
#else
    if (is_f2e()) {
#endif
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, 0xbefff);
    }
    else {
        /*------------------------------------------------------------------------------------------*\
         * Need to set the Out-of-range bit to receive the packets with 1518 bytes, when VLAN is enabled.
        \*------------------------------------------------------------------------------------------*/
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, 0x66bc2);
    }
#else
    /*--- athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, 0x3ffff); ---*/       /*--- erstmal alles empfangen ---*/
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, ATHR_DEFAULT_FRAMES);
#endif

    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TXFIFO_THRESH ,0x01d80160);
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_3, fifo_3);

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,ATHR_JUMBO_FR)) {
        athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG2, ATHR_GMAC_CFG2_HUGE_FRAME_EN);
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MAX_PKTLEN, frame_sz);
    }
#endif

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac, WAN_QOS_SOFT_CLASS)) {
        athr_gmac_reg_wr(mac->BaseAddress,ATHR_GMAC_DMA_TX_ARB_CFG,ATHR_GMAC_TX_QOS_MODE_WEIGHTED
                | ATHR_GMAC_TX_QOS_WGT_0(0x7)
                | ATHR_GMAC_TX_QOS_WGT_1(0x5) 
                | ATHR_GMAC_TX_QOS_WGT_2(0x3)
                | ATHR_GMAC_TX_QOS_WGT_3(0x1));

        for(ac = 0;ac < mac->mac_noacs; ac++) {
            tx = &mac->mac_txring[ac];
            t0  =  &tx->ring_desc[0];
            switch(ac) {
                case ENET_AC_VO:
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TX_DESC_Q0, athr_gmac_desc_dma_addr(tx, t0));
                    break;
                case ENET_AC_VI:
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TX_DESC_Q1, athr_gmac_desc_dma_addr(tx, t0));
                    break;
                case ENET_AC_BK:
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TX_DESC_Q2, athr_gmac_desc_dma_addr(tx, t0));
                    break;
                case ENET_AC_BE:
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TX_DESC_Q3, athr_gmac_desc_dma_addr(tx, t0));
                    break;
            }
        }
    } else {
#else 
    {
#endif
        tx = &mac->mac_txring[0];
        t0 = &tx->ring_desc[0];
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TX_DESC_Q0, athr_gmac_desc_dma_addr(tx, t0));
    }

    r0 = &rx->ring_desc[0];
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_RX_DESC, athr_gmac_desc_dma_addr(rx, r0));

    AVMNET_TRC("[%s] : cfg1 %#x cfg2 %#x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_CFG1), athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_CFG2));

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_setup_irq(avmnet_module_t *this, unsigned int on) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;

    AVMNET_INFOTRC("[%s] %s %s\n", __func__, this->name, on ? "on":"off");
    if (on) {
        athr_gmac_int_enable(mac->BaseAddress); 
    } else {
        athr_gmac_int_disable(mac->BaseAddress);
    }

    return 0;
}

#if 0
/*------------------------------------------------------------------------------------------*\
 * device functions
\*------------------------------------------------------------------------------------------*/
int athr_gmac_open(struct net_device *device) {

    avmnet_device_t *avm_device = ((avmnet_netdev_priv_t *)netdev_priv(device))->avm_dev;
    avmnet_module_t *this;
    int result;

    if (!avm_device) {
        printk("[%s] no avm_device found\n", __func__);
        return -EFAULT;
    }

    AVMNET_DEBUG("[%s] device %s\n", __func__, device->name);

    device->trans_start = jiffies;

    netif_carrier_off(device);
    netif_start_queue(device);

    this = avmnet_cfg_get_module(device);
    if (this) { 
        AVMNET_DEBUG("[%s] device %s %s\n", __func__, device->name, this->name);
        this->powerup(this);
        result = this->setup_irq(this, 1);
        if (result > 0) {
            // avmnet_timer_add(this, poll);  /*--- kein Interrupt möglich - Timer benutzten ---*/
        } else if(result < 0) {
            AVMNET_ERR("[%s] ERROR setup IRQ on %s\n", __func__, this->name);
            return -EINVAL;
        }
        return 0;
    } 

    AVMNET_ERR("[%s] <no device %s found>\n", __func__, device->name);
    return -EINVAL;

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athr_gmac_stop(struct net_device *device) {
    avmnet_module_t *this = avmnet_cfg_get_module(device);

    if (this) { 
        AVMNET_DEBUG("[%s] device %s %s\n", __func__, device->name, this->name);
        this->setup_irq(this, 0);
        this->powerdown(this);
        // avmnet_timer_del(this, poll);  /*--- kein Interrupt möglich - Timer benutzten ---*/
    } 

    netif_stop_queue(device);
    netif_carrier_off(device);

    return 0;
}
#endif

#define QUEUE_EMPTY(head, tail, anzahl)      ((head) == (tail))
#define QUEUE_FILL(head, tail, anzahl)       ((head) > (tail) ? ((head) - (tail)) : ((anzahl) + 1) - (tail) + (head))
#define QUEUE_FREE(head, tail, anzahl)       (((anzahl) + 1) - 1 - QUEUE_FILL(head, tail, anzahl))

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline int athr_gmac_ndesc_unused(athr_gmac_ring_t *ring) {
    int head = ring->ring_head, tail = ring->ring_tail;

    return ((tail > head ? 0 : ring->ring_nelem) + tail - head);
    /*--- return QUEUE_FREE(head, tail, ring->ring_nelem); ---*/
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline int athr_gmac_tx_reap_thresh(avmnet_module_t *this, int ac) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;
    athr_gmac_ring_t *r = &mac->mac_txring[ac];

    return (athr_gmac_ndesc_unused(r) < mac->reap_thresh);
    /*--- return QUEUE_FREE(r->ring_head, r->ring_tail, r->ring_nelem) < mac->reap_thresh; ---*/
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline int athr_gmac_tx_ring_full(avmnet_module_t *this,int ac) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;
    athr_gmac_ring_t *r = &mac->mac_txring[ac];

#if (AVMNET_USE_DEBUG_LEVEL_DEBUG != 0)
    if (athr_gmac_ndesc_unused(r) < 10)
        printk(KERN_ERR "{%s} ndesc_unused %d head %d tail %d\n", __func__, athr_gmac_ndesc_unused(r), r->ring_head, r->ring_tail);
#endif
    return (athr_gmac_ndesc_unused(r) < tx_max_desc_per_ds_pkt);
    /*--- return QUEUE_FREE(r->ring_head, r->ring_tail, r->ring_nelem) == 0; ---*/
}

/*------------------------------------------------------------------------------------------*\
 * Reap from tail till the head or whenever we encounter an unxmited packet.
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_tx_reap(avmnet_module_t *this, int ac) {    

    struct athmac_context *mac = (struct athmac_context *)this->priv;
    avmnet_device_t *avmnet_device;
    unsigned long flags;

    athr_gmac_ring_t   *r;
    athr_gmac_desc_t   *ds;
    int head, tail, i;
    unsigned int tx_status = athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_STATUS);
    unsigned int sent = (tx_status >> 16) & 0xFF;  /*--- gesendete Pakete ---*/

    spin_lock_irqsave(&mac->spinlock, flags);

    r = &mac->mac_txring[ac];
    head = r->ring_head;
    tail = r->ring_tail;

    AVMNET_DEBUG("[%s] head %d tail %d DMA_TX_STAT 0x%x sent %d (%d pairs)\n", __func__, head, tail, tx_status, sent, sent >> 1);

    while(tail != head) {
        ds = &r->ring_desc[tail * 2];

        if(athr_gmac_tx_owned_by_dma(ds)) {
            AVMNET_DEBUG("[%s] header desc not ready (tail %d)\n", __FUNCTION__, tail);
            break;
        }
        if(athr_gmac_tx_owned_by_dma(ds + 1)) {
            AVMNET_DEBUG("[%s] data desc not ready (tail %d)\n", __FUNCTION__, tail);
            break;
        }
        AVMNET_DEBUG("[%s] acc desc (tail %d) \n", __FUNCTION__, tail);

        athr_store_skb(r->ring_buffer[tail].buf_pkt);
        r->ring_buffer[tail].buf_pkt = NULL;
        athr_gmac_intr_ack_tx(mac->BaseAddress);
        sent--;
        athr_gmac_intr_ack_tx(mac->BaseAddress);
        sent--;
        athr_gmac_ring_incr(tail);
    }
    /*--- printk("*\n"); ---*/

    r->ring_tail = tail;

    AVMNET_DEBUG("[%s] sent %d  (register %d)\n", __FUNCTION__, sent, (athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_STATUS) >> 16) & 0xFF);

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,WAN_QOS_SOFT_CLASS) && (athr_gmac_ndesc_unused(r) >= mac->qstart_thresh) && netif_carrier_ok(mac->mac_dev)) {
            mac->tx_ac_bt &= ~(1 << ac);
    }
#endif

    if (mac->dma_stopped) {
        if (athr_gmac_ndesc_unused(r) >= mac->qstart_thresh) {
            /* Keep reaping till all the queues have enough descriptors.  */
#if defined(ATH_FLAGS)
            if (mac_has_flag(mac,WAN_QOS_SOFT_CLASS) && (mac->tx_ac_bt != 0)) {
                athr_gmac_intr_enable_tx(mac->BaseAddress);
                goto done;
            }
#endif
            mac->dma_stopped = 0;

            for (i = 0; i < this->num_net_devices; i++) {
                avmnet_device = this->net_devices[i];
                if (netif_queue_stopped(avmnet_device->device) && netif_carrier_ok(avmnet_device->device)) {
                    AVMNET_DEBUG("{%s} wakeup device %s\n", __func__, avmnet_device->device->name);
                    netif_tx_wake_all_queues(avmnet_device->device);
                }
            }
        } else {
            AVMNET_DEBUG("{%s} enable Interrupt unused Buffer %d\n", __func__, athr_gmac_ndesc_unused(r));
            athr_gmac_intr_enable_tx(mac->BaseAddress);
        }
    }

    spin_unlock_irqrestore(&mac->spinlock, flags);
    
    AVMNET_DEBUG("[%s] tx_status 0x%x tx_enable 0x%x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_STATUS), athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_CTRL));

#if defined(ATH_FLAGS)
done:
#endif
    return;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline athr_gmac_desc_t *athr_gmac_get_tx_ds(struct athmac_context *mac, struct sk_buff *skb, int skb_priority) {

    athr_gmac_desc_t      *ds;
    athr_gmac_ring_t      *r   = &mac->mac_txring[skb_priority];
#if defined(ATH_FLAGS)
    athr_gmac_buffer_t    *bp;
#endif

    /*--- printk("{%s} ring_head 0x%d\n", __func__, r->ring_head); ---*/
    ds = &r->ring_desc[r->ring_head * 2];  /* immer paarweise */
    ds->pkt_start_addr = (unsigned long)&(r->mac_txheader[r->ring_head]);
    r->ring_buffer[r->ring_head].buf_pkt = skb;

    /*--- BUG_ON(athr_gmac_tx_owned_by_dma(ds)); ---*/
    if (athr_gmac_tx_owned_by_dma(ds)) {
        panic("get_tx_ds desc athr_gmac_tx_owned_by_dma\n");
    }

    /*--- Time stamp each packet ---*/ 
#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,CHECK_DMA_STATUS)) {
        bp = &r->ring_buffer[r->ring_head];
        bp->trans_start = jiffies; 
    }
#endif


    athr_gmac_ring_incr(r->ring_head);

    return ds;
}

/*
 * Tx operation:
 * We do lazy reaping - only when the ring is "thresh" full. If the ring is 
 * full and the hardware is not even done with the first pkt we q'd, we turn
 * on the tx interrupt, stop all q's and wait for h/w to
 * tell us when its done with a "few" pkts, and then turn the Qs on again.
 *
 * Locking:
 * The interrupt only touches the ring when Q's stopped  => Tx is lockless, 
 * except when handling ring full.
 *
 * Desc Flushing: Flushing needs to be handled at various levels, broadly:
 * - The DDr FIFOs for desc reads.
 * - WB's for desc writes.
 */
static void athr_gmac_handle_tx_full(avmnet_module_t *this, int ac __attribute__((unused))) {

    avmnet_device_t *avmnet_device;
    struct athmac_context *mac = (struct athmac_context *)this->priv;
    int i;

    AVMNET_DEBUG("[%s]: %s %d\n", __func__, this->name, this->num_net_devices);

    /*--- Do nothing if the link went down while we are here. ---*/ 
    if (mac->dma_stopped) {
        AVMNET_DEBUG("[%s] %s queue always stoped\n", __func__, this->name);
        return;
    }

    mac->dma_stopped = 1;

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,WAN_QOS_SOFT_CLASS)) {
        mac->tx_ac_bt |=  (1 << ac);
        if (!((mac->tx_ac_bt & 0xf) == 0xf))
            return;
    }
#endif

    for (i = 0; i < this->num_net_devices; i++) {
        avmnet_device = this->net_devices[i];
        if (!netif_queue_stopped(avmnet_device->device)) {
            AVMNET_DEBUG("{%s} %s stop queue\n", __func__, avmnet_device->device->name);
            netif_tx_stop_all_queues(avmnet_device->device);
        } else {
            AVMNET_DEBUG("{%s} %s queue always stoped\n", __func__, avmnet_device->device->name);
        }
        avmnet_device->device->stats.tx_fifo_errors ++;
    }
    athr_gmac_intr_enable_tx(mac->BaseAddress);

}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athr_gmac_hard_start(struct sk_buff *skb, struct net_device *device) {

    avmnet_device_t    *avm_device = ((avmnet_netdev_priv_t *)netdev_priv(device))->avm_dev;
    avmnet_module_t          *this = (avmnet_module_t *)avm_device->mac_module;
    struct athmac_context     *mac = (struct athmac_context *)this->priv;

    athr_gmac_ring_t    *r;
    athr_gmac_desc_t    *ds, *head_ds;
    unsigned int        ac = 0;
    unsigned long       flags;
    athr_mac_packet_header_t *header;

    /*--------------------------------------------------------------------------------------*\
     * TX DMA Descriptoren aufräumen
    \*--------------------------------------------------------------------------------------*/
    for (ac = 0;ac < mac->mac_noacs; ac++) {
        if (athr_gmac_tx_reap_thresh(this, ac)) {
            AVMNET_DEBUG("[%s] name %s\n", __func__, this->name);
            athr_gmac_tx_reap(this ,ac);
        }
    }

    if(skb->len < 16) {
        AVMNET_INFO("[%s] skb->len %u < 16. Dropping skb %p.\n", __func__, skb->len, skb);
        goto dropit;
    }
    skb->priority = ENET_AC_BE;

    spin_lock_irqsave(&mac->spinlock, flags);

    r = &mac->mac_txring[skb->priority];

    if (!r) {
        AVMNET_ERR("[%s] skb->priority %d\n", __func__, skb->priority);
        /*--- BUG_ON(1); ---*/
        panic("!r");
    }

    athr_mac_dma_cache_sync(skb->data, skb->len);

#if 0
    athr_gmac_tx_qos(mac,skb);

    if (athr_gmac_check_qos_bt(mac,skb))
        goto dropit;

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,ATHR_HEADER_ENABLED))
        athr_gmac_add_hdr(mac,skb);
#endif
#endif


#if (AVMNET_USE_DEBUG_LEVEL_DEBUG != 0)
    printk(KERN_ERR "sent - h %d t %d l %d 0x%x 0x%x 0x%x 0x%x\n", 
                        r->ring_head, 
                        r->ring_tail, 
                        skb->len, 
                        athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_STATUS), 
                        athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_CTRL), 
                        athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_FIFO_DEPTH), 
                        athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TXFIFO_THRESH));
#endif

    switch (mac->mac_unit) {
        case 0:
            /*------------------------------------------------------------------------------*\
             * DMA Descriptor für den MAC Header und den Atheros Spezial Header (14 Bytes)   1.
             * DMA Descriptor für den Inhalt des Paketes ohne MAC Header (offset 12 Bytes)   2.
            \*------------------------------------------------------------------------------*/
            head_ds = athr_gmac_get_tx_ds(mac, skb, skb->priority);
            head_ds->res1           = 0;
            head_ds->more           = 1;
            head_ds->res2           = 0;
            head_ds->ftpp_override  = 0;
            head_ds->res3           = 0;

            header = (athr_mac_packet_header_t *)head_ds->pkt_start_addr;

            memcpy(header->buff, skb->data, 2 * ETH_ALEN);

            head_ds->pkt_size         = 2 * ETH_ALEN;
            head_ds->pkt_start_addr   = virt_to_phys(header);

            ds                        = head_ds + 1;
            ds->res1                  = 0;
            ds->more                  = 0;
            ds->res2                  = 0;
            ds->ftpp_override         = 0;
            ds->res3                  = 0;

            ds->pkt_size              = skb->len - 2 * ETH_ALEN;
            ds->pkt_start_addr        = virt_to_phys((skb->data + 2 * ETH_ALEN));
            athr_mac_dma_cache_sync((unsigned long)header, 2 * ETH_ALEN);
            athr_mac_dma_cache_sync(skb->data + 2 * ETH_ALEN, skb->len - 2 * ETH_ALEN);
            break;
        case 1:
            /*------------------------------------------------------------------------------*\
             * DMA Descriptor für den MAC Header und den Atheros Spezial Header (14 Bytes)   1.
             * DMA Descriptor für den Inhalt des Paketes ohne MAC Header (offset 12 Bytes)   2.
            \*------------------------------------------------------------------------------*/
            head_ds = athr_gmac_get_tx_ds(mac, skb, skb->priority);
            head_ds->res1           = 0;
            head_ds->more           = 1;
            head_ds->res2           = 0;
            head_ds->ftpp_override  = 0;
            head_ds->res3           = 0;

            header = (athr_mac_packet_header_t *)head_ds->pkt_start_addr;

            memcpy(header->buff, skb->data, 2 * ETH_ALEN);

            header->s.header.version  = 2;
            header->s.header.priority = skb->priority;
            header->s.header.type     = 0;
            header->s.header.from_cpu = 1;
            header->s.header.portnum  = 1 << (avm_device->vlanID); 
            header->s.data            = *(unsigned short *)(skb->data + 2 * ETH_ALEN);

            head_ds->pkt_size         = 2 * ETH_ALEN + 2 + 2;
            head_ds->pkt_start_addr   = virt_to_phys(header);

            ds                        = head_ds + 1;
            ds->res1                  = 0;
            ds->more                  = 0;
            ds->res2                  = 0;
            ds->ftpp_override         = 0;
            ds->res3                  = 0;

            ds->pkt_size              = skb->len - 2 * ETH_ALEN - 2;
            ds->pkt_start_addr        = virt_to_phys((skb->data + 2 * ETH_ALEN + 2));
            athr_mac_dma_cache_sync((unsigned long)header, 2 * ETH_ALEN + 2);
            athr_mac_dma_cache_sync(skb->data + 2 * ETH_ALEN + 2, skb->len - 2 * ETH_ALEN - 2);
            break;
        default:
            panic("illegal mac unit\n");
    }

/*--- #ifdef CONFIG_ATHRS_HW_NAT ---*/
    /*--- athr_nat_process_egress_pkts(mac->mac_unit, skb, ds); ---*/
/*--- #endif ---*/

/*--- #ifdef CONFIG_ATHRS_HW_ACL ---*/
    /*--- athr_acl_enable_tx(mac,ds); ---*/
/*--- #endif ---*/

    wmb();

    avm_device->device->stats.tx_packets ++;
    avm_device->device->stats.tx_bytes += skb->len;

    /*--------------------------------------------------------------------------------------*\
     * Bei zweigeteilten Frames zuerst den Zweiten Teil auf DMA Owner setzen und anschließend
     * den ersten. Wenn man das umgekehrt macht, besteht die Gefahr, dass zwischen die beiden
     * 'give_to_dma' aufrufe ein Interrupt kommt und den Vorgang verzögert. Dann könnte der
     * DMA Controller schon mit dem Senden des ersten Teil angefangen haben und findet kein 
     * zweiten Teil. Dadurch wird der Senden des gesamtframe unterbrochen und somit der Frame
     * zerstört.
    \*--------------------------------------------------------------------------------------*/
    athr_gmac_tx_give_to_dma(ds);       
    athr_gmac_tx_give_to_dma(head_ds);

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,WAN_QOS_SOFT_CLASS)) {
        athr_gmac_tx_start_qos(mac,skb->priority);
    } else {
        athr_gmac_tx_start(mac);
    }
#else
    if ( ! athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_TX_CTRL)) {
        athr_gmac_tx_start(mac->BaseAddress);
    }
#endif


    for ( ac = 0;ac < mac->mac_noacs; ac++) {
        if (unlikely(athr_gmac_tx_ring_full(this, ac))) {
            athr_gmac_handle_tx_full(this, ac);
        }
    }

    spin_unlock_irqrestore(&mac->spinlock, flags);

    device->trans_start = jiffies;

    return NETDEV_TX_OK;

dropit:
#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,WAN_QOS_SOFT_CLASS))
        mac->tx_ac_drops++;
    else
#endif
    kfree_skb(skb);

    return NETDEV_TX_OK;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void athr_gmac_tx_timeout(struct net_device *device) {

    avmnet_device_t *avm_device = ((avmnet_netdev_priv_t *)netdev_priv(device))->avm_dev;
    avmnet_module_t       *this = (avmnet_module_t *)avm_device->mac_module;
    struct athmac_context *mac = (struct athmac_context *)this->priv;

    printk(KERN_ERR "[%s] dev %s\n", __func__, device->name);

    /*--- Do the reset outside of interrupt context ---*/ 
#if 0
    schedule_work(&mac->mac_tx_timeout);
#endif

    tasklet_schedule(&mac->txreaptq);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if 0
static void athr_gmac_tx_timeout_task(struct work_struct *ws) {  

    struct athmac_context *mac = (struct athmac_context *)container_of(ws, struct athmac_context, mac_tx_timeout);

    AVMNET_WARN("[%s]\n", __func__);

    /*--- panic("athr_gmac_tx_timeout\n"); ---*/
    if (mac->ops->check_dma_st)
       mac->ops->check_dma_st(mac,0);

}
#endif

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline void athr_gmac_set_mac_duplex(struct athmac_context *mac, int fdx) {
    if (fdx) {
        athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG2, ATHR_GMAC_CFG2_FDX);
    } else {
        athr_gmac_reg_rmw_clear(mac->BaseAddress, ATHR_GMAC_CFG2, ATHR_GMAC_CFG2_FDX);
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline void athr_gmac_set_mac_speed(struct athmac_context *mac, int speed) {
    
    switch (speed) {
        case ATHR_PHY_SPEED_10T: 
            athr_gmac_reg_rmw_clear(mac->BaseAddress, ATHR_GMAC_IFCTL, ATHR_GMAC_IFCTL_SPEED);
            break;
        case ATHR_PHY_SPEED_100T:
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_IFCTL, ATHR_GMAC_IFCTL_SPEED);
            break;
        case ATHR_PHY_SPEED_1000T:
        case ATHR_PHY_SPEED_UNKNOWN:
            break;
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static inline void athr_gmac_set_mac_if(struct athmac_context *mac, int speed) {

    athr_gmac_reg_rmw_clear(mac->BaseAddress, ATHR_GMAC_CFG2, (ATHR_GMAC_CFG2_IF_1000| ATHR_GMAC_CFG2_IF_10_100));

    switch (speed) {
        case ATHR_PHY_SPEED_10T: 
        case ATHR_PHY_SPEED_100T:
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG2, ATHR_GMAC_CFG2_IF_10_100);
            athr_gmac_reg_rmw_clear(mac->BaseAddress,ATHR_GMAC_FIFO_CFG_5, ATHR_GMAC_BYTE_PER_CLK_EN);
            break;
        case ATHR_PHY_SPEED_1000T:
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG2, ATHR_GMAC_CFG2_IF_1000);
            athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5, ATHR_GMAC_BYTE_PER_CLK_EN);
            break;
        case ATHR_PHY_SPEED_UNKNOWN:
            break;
    }

}

/*------------------------------------------------------------------------------------------*\
 * Several fields need to be programmed based on what the PHY negotiated
 * Ideally we should quiesce everything before touching the pll, but:
 * 1. If its a linkup/linkdown, we dont care about quiescing the traffic.
 * 2. If its a single gigE PHY, this can only happen on lup/ldown.
 * 3. If its a 100Mpbs switch, the link will always remain at 100 (or nothing)
 * 4. If its a gigE switch then the speed should always be set at 1000Mpbs, 
 *    and the switch should provide buffering for slower devices.
 *
 * XXX Only gigE PLL can be changed as a parameter for now. 100/10 is hardcoded.
 * XXX Need defines for them -
 * XXX FIFO settings based on the mode
\*------------------------------------------------------------------------------------------*/
void athr_gmac_set_mac_from_link(avmnet_module_t *this, avmnet_linkstatus_t status) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;

    if (mac->mac_unit == 0) {
        athr_gmac_set_mac_if(mac, status.Details.speed);
        athr_gmac_set_mac_speed(mac, status.Details.speed);
        athr_gmac_set_mac_duplex(mac, status.Details.fullduplex);

        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_3, fifo_3);
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TXFIFO_THRESH ,0x01d80160);

        switch (status.Details.speed) {
            case ATHR_PHY_SPEED_1000T: 
                AVMNET_INFO("[%s] ATHR_PHY_SPEED_1000T\n", __func__);

#if defined(CONFIG_MACH_QCA955x)
                ath_reg_wr(ATH_ETH_XMII_CONFIG, ETH_XMII_GIGE_SET(1) | ETH_XMII_TX_INVERT_SET(1));
#else
                ath_reg_wr(ATH_ETH_XMII_CONFIG, ETH_XMII_GIGE_SET(1));
#endif
#if 0
                if (is_ar7242())
                    ath_reg_wr(ATH_ETH_XMII_CONFIG, (1<<28)|(1<<26)|(1<<25));

                if (is_ar934x() && (is_s16() || is_s17()))
                    ath_reg_wr(ATH_ETH_XMII_CONFIG, (1<<26)|(1<<25));

                if (is_ar934x() && is_f1e() )
                    ath_reg_wr(ATH_ETH_XMII_CONFIG, (1<<28)|(1<<25));
#endif
                break;

            case ATHR_PHY_SPEED_100T:
                AVMNET_INFO("[%s] ATHR_PHY_SPEED_100T\n", __func__);

                if (! status.Details.fullduplex) {
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TXFIFO_THRESH ,0x00880060);
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_3, 0x00f00040);
                }

#if defined(CONFIG_MACH_QCA955x)
                ath_reg_wr(ATH_ETH_XMII_CONFIG,  
                        ETH_XMII_TX_INVERT_SET(1)|ETH_XMII_PHASE0_COUNT_SET(0x01)|ETH_XMII_PHASE1_COUNT_SET(0x01));
#else
                ath_reg_wr(ATH_ETH_XMII_CONFIG,  
                        ETH_XMII_PHASE0_COUNT_SET(0x01)|ETH_XMII_PHASE1_COUNT_SET(0x01));
#endif
                break;

            case ATHR_PHY_SPEED_10T:
                AVMNET_INFO("[%s] ATHR_PHY_SPEED_10T\n", __func__);

                if (! status.Details.fullduplex) {
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_DMA_TXFIFO_THRESH ,0x00880060);
                    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_3, 0x00f00040);
                }

#if defined(CONFIG_MACH_QCA955x)
                ath_reg_wr(ATH_ETH_XMII_CONFIG, 
                        ETH_XMII_TX_INVERT_SET(1)|ETH_XMII_PHASE0_COUNT_SET(0x13)|ETH_XMII_PHASE1_COUNT_SET(0x13));
#else
                ath_reg_wr(ATH_ETH_XMII_CONFIG, 
                        ETH_XMII_PHASE0_COUNT_SET(0x13)|ETH_XMII_PHASE1_COUNT_SET(0x13));
#endif
                break;

            default:
                AVMNET_INFO("[%s] ATHR_PHY_SPEED_UNKNOWN\n", __func__);
                BUG_ON(1);
        }
    } 

    AVMNET_DEBUG("[%s]: cfg_1: %#x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_1));
    AVMNET_DEBUG("[%s]: cfg_2: %#x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_2));
    AVMNET_DEBUG("[%s]: cfg_3: %#x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_3));
    AVMNET_DEBUG("[%s]: cfg_4: %#x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_4));
    AVMNET_DEBUG("[%s]: cfg_5: %#x\n", __func__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_FIFO_CFG_5));
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athmac_reg_busy_wait(avmnet_module_t *this) {
    struct athmac_context *mac = (struct athmac_context *)this->priv;
    int rddata;
    unsigned short ii = 0x1000;

    do {
        schedule();
        rddata = athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_MII_MGMT_IND) & 0x7;
    } while (rddata && --ii);

    if (ii == 0) {
        AVMNET_ERR("ERROR:%s:%d transaction failed\n",__func__,__LINE__);
    }
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int athmac_reg_read(avmnet_module_t *this, unsigned int phyAddr, unsigned int reg) {
    struct athmac_context *mac = (struct athmac_context *)this->priv;
    unsigned int addr = (phyAddr << ATHR_GMAC_ADDR_SHIFT) | reg, val = 0;

    /*--- printk(KERN_ERR "[%s/%d] unit 0x%x phy_addr 0x%x reg_addr 0x%x\n", __func__, __LINE__, mac, phyAddr, reg); ---*/

    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_ADDRESS, addr);
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_CMD, ATHR_GMAC_MII_MGMT_CMD_READ);

    athmac_reg_busy_wait(this);

    val = athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_MII_MGMT_STATUS);
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_CMD, 0x0);

    athmac_reg_busy_wait(this);

    return val;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_reg_write(avmnet_module_t *this, unsigned int phyAddr, unsigned int reg, unsigned int Value) {
    struct athmac_context *mac = (struct athmac_context *)this->priv;
    unsigned short addr  = (phyAddr << ATHR_GMAC_ADDR_SHIFT) | reg;

    /*--- AVMNET_DEBUG("[%s/%d] count %u unit 0x%x phy_addr 0x%x reg_addr 0x%x value 0x%x\n", __func__, __LINE__, mdio_count, mac, phyAddr, reg, Value); ---*/

    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_ADDRESS, addr);
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_CTRL, Value);

    athmac_reg_busy_wait(this);

    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_lock(avmnet_module_t *this) {
    struct athmac_context *mac = (struct athmac_context *)this->priv;

    /*--- AVMNET_DEBUG("[%s]\n", __func__); ---*/
    return down_interruptible(&mac->lock);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void athmac_unlock(avmnet_module_t *this) {
    struct athmac_context *mac = (struct athmac_context *)this->priv;

    /*--- AVMNET_DEBUG("[%s]\n", __func__); ---*/
    up(&mac->lock);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void athmac_status_changed(avmnet_module_t *this, avmnet_module_t *caller) {

    avmnet_module_t *child = NULL;
    int i;

    /*
     * find out which of our progeny demands our attention
     */
    for(i = 0; i < this->num_children; ++i){
        if(this->children[i] == caller){
            child = this->children[i];
            break;
        }
    }

    if(child == NULL) {
        AVMNET_ERR("[%s] module %s: received status_changed from unknown module.\n", __func__, this->name);
        return;
    }

    /*
     * handle status change
     */

    return;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_poll(avmnet_module_t *this __attribute__ ((unused))) {
    int i, result;

    for(i = 0; i < this->num_children; ++i){
        result = this->children[i]->poll(this->children[i]);
        if(result < 0){
            AVMNET_WARN("Module %s: poll() failed on child %s\n", this->name, this->children[i]->name);
        }
    }

    return 0;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athgmac_set_status(avmnet_module_t *this, avmnet_device_t *id, avmnet_linkstatus_t status) {
    struct athmac_context *mac;
    struct net_device *device;
    unsigned long flags;

    /* FIXME Added on 2012, Jun 8th; asserts should be removed later on */
    /* Should be fixed since 2012, June 20th */
    assert(this != NULL);
    assert(id != NULL);
    device = id->device;
    assert(device != NULL);

    mac = (struct athmac_context *) this->priv;

    AVMNET_DEBUG("[%s] %s - %s: carrier %s link %d status 0x%x\n", __func__, this->name, device->name, status.Details.link ? "on":"off", status.Details.link, status.Status);

    id->status = status;

    avmnet_links_port_update(id);

    if (status.Details.link) {      /*--- link up ---*/

        if(!(this->initdata.mac.flags & AVMNET_CONFIG_FLAG_SWITCHPORT)){
            athr_gmac_set_mac_from_link(this, status);      /*--- set Interface ---*/
        }else{
            AVMNET_DEBUG("[%s] %s: Connected to switch, not touching interface config.\n", __func__, this->name);
        }

        if(!netif_carrier_ok(device)){
            AVMNET_DEBUG("[%s] %s: setting carrier on\n", __func__, device->name);
            netif_carrier_on(device);
        }

        spin_lock_irqsave(&mac->spinlock, flags);

        if(netif_queue_stopped(device) && !mac->dma_stopped){
            AVMNET_DEBUG("[%s] %s: starting tx queues\n", __func__, device->name);
            netif_tx_wake_all_queues(device);
        }

        spin_unlock_irqrestore(&mac->spinlock, flags);

    } else {                        /*--- link down ---*/
        if(!(this->initdata.mac.flags & AVMNET_CONFIG_FLAG_SWITCHPORT)){
            athr_gmac_intr_disable_tx(mac->BaseAddress);    /*--- disable TX ---*/
        }else{
            AVMNET_DEBUG("[%s] %s: Connected to switch, not disabling MAC.\n", __func__, this->name);
        }

        if(netif_carrier_ok(device)) {
            AVMNET_DEBUG("[%s] %s: seting carrier off, stopping tx queues\n", __func__, device->name);
            netif_carrier_off(device);
            netif_tx_stop_all_queues(device);
        }
    }

    return 0;
};

/*------------------------------------------------------------------------------------------*\
 * Head is the first slot with a valid buffer. Tail is the last slot replenished. 
 * Tries to refill buffers from tail to head.
\*------------------------------------------------------------------------------------------*/
static int athr_gmac_rx_replenish(avmnet_module_t *this) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;

    athr_gmac_ring_t   *r  = &mac->mac_rxring;
    int              head  = r->ring_head, tail = r->ring_tail, refilled = 0;
    athr_gmac_desc_t   *ds;
    athr_gmac_buffer_t *bf;

    /*--- AVMNET_DEBUG("[%s] name %s head %d tail %d\n", __func__, this->name, head, tail); ---*/

    /*--- while (!QUEUE_EMPTY(head, tail, size)) { ---*/
    while ( tail != head ) {
        bf = &r->ring_buffer[tail];
        ds = &r->ring_desc[tail];

#ifndef MEMLAT_OPT
        BUG_ON(athr_gmac_rx_owned_by_dma(ds));
#endif

        BUG_ON(bf->buf_pkt);

        bf->buf_pkt = athr_get_skb();   /*--- bf->buf_pkt = athr_gmac_buffer_alloc(); ---*/
        if (!bf->buf_pkt) {
            refilled = -1;
            AVMNET_ERR("[%s] : outta skbs!\n", __func__);
            break;
        }

        /* Invalidate cache should be chache aligned */
        /*--- athr_mac_cache_inv((unsigned long)(bf->buf_pkt->data - 2), ATHR_GMAC_RX_BUF_SIZE + frame_sz - ATHR_GMAC_RX_RESERVE); ---*/
        athr_mac_cache_inv((unsigned long)(bf->buf_pkt->data), ATHR_GMAC_RX_BUF_SIZE + frame_sz);

        ds->pkt_start_addr  = virt_to_phys(bf->buf_pkt->data);

        athr_gmac_rx_give_to_dma(ds);
        refilled ++;

        athr_gmac_ring_incr(tail);
    }

    /* Flush descriptors */
    wmb();

    if (is_ar7240() || is_ar933x() || is_qca955x()) {
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_CFG1, (athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_CFG1) | 0xc));
        athr_gmac_rx_start(this);
    }

    r->ring_tail = tail;

    return refilled;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_txreap_task(unsigned long data) {

    avmnet_module_t *this = (avmnet_module_t *)data;
    struct athmac_context *mac = (struct athmac_context *)this->priv;
    unsigned int ac;

    /*--- AVMNET_ERR("[%s] %s\n", __func__, this->name); ---*/
    for(ac = 0; ac < mac->mac_noacs; ac++)
        athr_gmac_tx_reap(this, ac);

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int athr_gmac_recv_packets(struct net_device *dev, avmnet_module_t *this, int quota, int *work_done) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;

    athr_gmac_ring_t       *r     = &mac->mac_rxring;
    athr_gmac_desc_t       *ds;
    struct sk_buff         *skb;
    athr_gmac_rx_status_t   ret   = ATHR_GMAC_RX_STATUS_DONE;
    int head = r->ring_head, len, status, more_pkts;
    int start_quota = quota;
    int replenished = 0;
#ifdef MEMLAT_OPT
    athr_gmac_desc_t       ds_pre[2];
    int head_pre;
#endif

    status = athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS);
    /*--- AVMNET_DEBUG("[%s] %s status 0x%x quota %d\n", __func__, this->name, status, quota); ---*/

process_pkts:

    BUG_ON( ! (quota > 0)); 

    if (mac->dma_check) {
        mac->dma_check = 0;
    } else if ((status & ATHR_GMAC_RX_STATUS_PKT_RCVD) == 0) {
        printk(KERN_ERR "[%s] exit ATHR_GMAC_RX_STATUS_PKT_RCVD == 0\n", __FUNCTION__);
        goto done;
    }

#ifdef MEMLAT_OPT
    head_pre = head;
    ds_pre[quota & 1] = r->ring_desc[head_pre];
#endif

    while(quota) {
#ifdef MEMLAT_OPT
        ds = ds_pre + (quota & 1);
        if (quota - 1) {
            athr_gmac_ring_incr(head_pre);
            ds_pre[(quota & 1) ^ 1] = r->ring_desc[head_pre];
        }
#else
        ds    = &r->ring_desc[head];
#endif

        if (athr_gmac_rx_owned_by_dma(ds)) {
            BUG_ON(quota == start_quota);
            break;
        }
        athr_gmac_intr_ack_rx(mac->BaseAddress);

        skb  = r->ring_buffer[head].buf_pkt;
        r->ring_buffer[head].buf_pkt = NULL;
        len = ds->pkt_size;
        quota--;

        BUG_ON(!skb);
        skb_put(skb, len - ETHERNET_FCS_SIZE);

        /*
         * Data corruption sometimes with large buffer size of 4k.Cached region 
         * is not synced with non cached region putting temporary fix for 
         * invalidating cache contents Need to find the root cause.
         */
        athr_mac_cache_inv((unsigned long)(skb->data), skb->len);

#if 0
        {
            typedef struct _atheros_special_header {
                unsigned short version  : 2;
                unsigned short priority : 2;
                unsigned short type     : 4;
                unsigned short from_cpu : 1;
                unsigned short portnum  : 7;
            } *p_atheros_special_header_t __attribute__ ((packed));
            p_atheros_special_header_t header;

            header = (p_atheros_special_header_t)(skb->data + 12);

            printk(KERN_ERR "type=%d port=0x%x from_cpu=%d\n", header->type, header->portnum, header->from_cpu);


            memmove(skb->data + 12, skb->data + 14, skb->len - 12);
            skb->len -= 2;
        }
#endif

        /*------------------------------------------------------------------------------------------*\
         * hier die VLANs auseinanderpflücken und in die Daten ins richtige Device stecken
        \*------------------------------------------------------------------------------------------*/
        skb_reset_mac_header(skb);
        switch(mac->mac_unit) {
            case 1:
                {
                    /*--- struct vlan_ethhdr *eth_head = vlan_eth_hdr(skb); ---*/
                    avmnet_module_t *ath_switch = this->children[0];
                    unsigned short vlan;
                    p_athr_special_header_t header;

                    header = (p_athr_special_header_t)(skb->data + 12);
                    vlan = header->portnum - 1;

                    if (vlan < ath_switch->num_children) {
                        dev = ath_switch->children[vlan]->device_id->device;
                    } else {
                        AVMNET_ERR("[%s] illegal port %d on %s\n", __func__, vlan, dev->name);
                        dev_kfree_skb_any(skb);
                        athr_gmac_ring_incr(head);
                        continue;
                    }
                    memmove(skb->data + 2, skb->data, 12);
                    skb_pull(skb, 2);

                    dev->stats.rx_packets++;
                    dev->stats.rx_bytes += skb->len;
                }
                break;
            case 0:
                {
                    dev = this->children[0]->device_id->device;
                    dev->stats.rx_packets++;
                    dev->stats.rx_bytes += skb->len;
                }
                break;
            default:
                panic("illegal mac unit\n");
        }
        
#if 0
        if(avm_pa_dev_receive(AVM_PA_DEVINFO(dev), skb) == 0)
            return CPMAC_ERR_NOERR; /* skb accelerated */
#endif

        /*--- AVMNET_DEBUG("[%s] dev %s\n", __func__, dev->name); ---*/
        skb->dev            = dev;
#       if defined(CONFIG_IP_MULTICAST_FASTFORWARD)
        {
            avmnet_netdev_priv_t *priv = netdev_priv(dev);
            mcfw_portset set;
            mcfw_portset_reset(&set);
            mcfw_portset_port_add(&set, 0);
            (void) mcfw_snoop_send(&priv->mcfw_netdrv, set, skb);
        }
#       endif /*--- #if defined(CONFIG_IP_MULTICAST_FASTFORWARD) ---*/
        skb->protocol       = eth_type_trans(skb, dev);
        dev->last_rx        = jiffies;

#ifdef CONFIG_ATHRS_HW_NAT
        athr_nat_process_ingress_pkts(mac->mac_unit, skb, ds);
#endif
        /*--- AVMNET_DEBUG("[%s] (%u) % *pB ... rx\n", __func__, skb->len, min(24u, skb->len), skb->data); ---*/
        netif_receive_skb(skb);

        if (mac->rx_dma_check) {
            mac->rx_dma_check = 0;
        }

        athr_gmac_ring_incr(head);
    }

    BUG_ON(start_quota == quota);
    r->ring_head   =  head;

    replenished = athr_gmac_rx_replenish(this);

    /*
    * let's see what changed while we were slogging.
    * ack Rx in the loop above is no flush version. It will get flushed now.
    */
    status       =  athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS);
    more_pkts    =  (status & ATHR_GMAC_RX_STATUS_PKT_RCVD);

    /*--- AVMNET_DEBUG("[%s] status 0x%x quota %d\n", __func__, status, quota); ---*/

    if (!more_pkts) goto done;
    /*--- more pkts arrived; if we have quota left, get rolling again ---*/
    if (quota)      goto process_pkts;
    /*--- out of quota ---*/

    ret = ATHR_GMAC_RX_STATUS_NOT_DONE;

done:
    *work_done   = (start_quota - quota);

    if (unlikely(replenished < 0)) {
        AVMNET_DEBUG("{ATHR_GMAC_RX_STATUS_OOM}\n");
        return ATHR_GMAC_RX_STATUS_OOM;
    }

    if (unlikely((ret == ATHR_GMAC_RX_STATUS_DONE) && ! (athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_CTRL)))) {
        athr_gmac_rx_start(mac->BaseAddress);
        AVMNET_DEBUG("[%s] RXE %d RX_DMA_STATUS 0x%x\n", __FUNCTION__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_CTRL), athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS));
    }

    /*--- !oom; if stopped, restart h/w ---*/
    if (unlikely(status & ATHR_GMAC_RX_STATUS_OVF)) {
        /*--- mac->net_rx_over_errors ++; ---*/     /*--- keine Statistik - es gibt kein passendes Device ---*/

        /*--- AVMNET_DEBUG("[%s] ATHR_GMAC_RX_STATUS_OVF\n", __FUNCTION__); ---*/
        athr_gmac_intr_ack_rxovf(mac->BaseAddress);
    }

#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,ATHR_RX_TASK)) 
        athr_gmac_intr_enable_recv(mac->BaseAddress);
#endif

    return ret;
}

/*------------------------------------------------------------------------------------------*\
 * Interrupt handling:
 * - Recv NAPI style (refer to Documentation/networking/NAPI)
 *
 *   2 Rx interrupts: RX and Overflow (OVF).
 *   - If we get RX and/or OVF, schedule a poll. Turn off _both_ interurpts. 
 *
 *   - When our poll's called, we
 *     a) Have one or more packets to process and replenish
 *     b) The hardware may have stopped because of an OVF.
 *
 *   - We process and replenish as much as we can. For every rcvd pkt 
 *     indicated up the stack, the head moves. For every such slot that we
 *     replenish with an skb, the tail moves. If head catches up with the tail
 *     we're OOM. When all's done, we consider where we're at:
 *
 *      if no OOM:
 *      - if we're out of quota, let the ints be disabled and poll scheduled.
 *      - If we've processed everything, enable ints and cancel poll.
 *
 *      If OOM:
 *      - Start a timer. Cancel poll. Ints still disabled. 
 *        If the hardware's stopped, no point in restarting yet. 
 *
 *      Note that in general, whether we're OOM or not, we still try to
 *      indicate everything recvd, up.
 *
 * Locking: 
 * The interrupt doesnt touch the ring => Rx is lockless
\*------------------------------------------------------------------------------------------*/
static irqreturn_t athr_gmac_intr(int irq __attribute__((unused)), void *dev) {
    
    avmnet_module_t *this = (avmnet_module_t *)dev;
    struct athmac_context *mac = (struct athmac_context *)this->priv;

    int   isr, imr, handled = 0;

    isr   = athr_gmac_get_isr(mac->BaseAddress);
    imr   = athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_INTR_MASK);

    /*--- AVMNET_DEBUG("[%s] %s isr 0x%x imr 0x%x\n", __func__, this->name, isr, imr); ---*/

    BUG_ON(isr != (isr & imr));

    if (likely(isr & ATHR_GMAC_INTR_RX)) {
        handled = 1;
        if (likely(napi_schedule_prep(&mac->napi))) {
            int status = athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS);
            athr_gmac_intr_disable_recv(mac->BaseAddress);
            BUG_ON(!(status & ATHR_GMAC_RX_STATUS_PKT_RCVD));
            BUG_ON(!(status >> 16));
            __napi_schedule(&mac->napi);     /*--- napi_schedule ohne Prüfung ---*/
        } else {
            AVMNET_ERR("[%s]: %s driver bug! interrupt while in poll isr 0x%x imr 0x%x\n", __func__, this->name, isr, imr);
            /*--- AVMNET_ERR("[%s]: epc = %pF athr_rx_disable_count %ld\n", this->name, (void *)read_c0_epc(), atomic_read(&athr_rx_disable_count)); ---*/
            athr_gmac_intr_disable_recv(mac->BaseAddress);
        }
    }
    if (unlikely(isr & (ATHR_GMAC_INTR_RX_OVF))) {
        handled = 1;
        athr_gmac_intr_ack_rxovf(mac->BaseAddress);
        AVMNET_DEBUG("[%s] RX_OVERRUN\n", __FUNCTION__);

        /*------------------------------------------------------------------------------------------*\
         * ist an dieser Stelle sinnlos
         * 1. der rxoverrun tritt sofort wieder auf
         * 2. Rx_Enable bleibt nicht gesetzt
        \*------------------------------------------------------------------------------------------*/
#if 0
        if (is_ar7240() || is_ar933x()) {
            athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_CFG1, (athr_gmac_reg_rd(mac->BaseAddress,ATHR_GMAC_CFG1) & 0xfffffff3));
        } else {
            athr_gmac_rx_start(mac->BaseAddress);
        }
#endif
    }
    if (likely(isr & ATHR_GMAC_INTR_TX)) {
        handled = 1;
        AVMNET_DEBUG("[%s] ATHR_GMAC_INTR_TX\n", __func__);
        athr_gmac_intr_disable_tx(mac->BaseAddress);
        tasklet_schedule(&mac->txreaptq);
    }
    if (unlikely(isr & ATHR_GMAC_INTR_RX_BUS_ERROR)) {
        BUG_ON(1);
        handled = 1;
        athr_gmac_intr_ack_rxbe(mac->BaseAddress);
    }
    if (unlikely(isr & ATHR_GMAC_INTR_TX_BUS_ERROR)) {
        BUG_ON(1);
        handled = 1;
        athr_gmac_intr_ack_txbe(mac->BaseAddress);
    }
    if (!handled) {
        /*
         * Mac fast reset will clear the status register if we get a 
         * previously queued interrupt after reset ignore it.
         */
        if (mac->dma_check) {
            mac->dma_check = 0;
            return IRQ_HANDLED;
        }
        AVMNET_ERR("[%s]: unhandled intr isr %#x\n", __func__, isr);
        BUG_ON(1);
    }

    return IRQ_HANDLED;
}

/*------------------------------------------------------------------------------------------*\
 * Error timers
\*------------------------------------------------------------------------------------------*/
static void athr_gmac_oom_timer(unsigned long data) {

    avmnet_module_t *this = (avmnet_module_t *)data;
    struct athmac_context *mac = (struct athmac_context *)this->priv;

    AVMNET_DEBUG("[%s] %s\n", __func__, this->name);
    if (athr_gmac_rx_replenish(this) < 0) {  /*--- OOM ---*/
        int val = mod_timer(&mac->mac_oom_timer, jiffies + 5);
        BUG_ON(val);
    } else {
        if ( ! (athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_CTRL))) {
            athr_gmac_rx_start(mac->BaseAddress);
            AVMNET_DEBUG("[%s] RXE %d RX_DMA_STATUS 0x%x\n", __FUNCTION__, athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_CTRL), athr_gmac_reg_rd(mac->BaseAddress, ATHR_GMAC_DMA_RX_STATUS));
        }
        napi_reschedule(&mac->napi);
        /*--- napi_schedule(&mac->napi); ---*/
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void athmac_gmac_setup_mii(struct athmac_context *mac) {
    unsigned int mgmt_cfg_val = 0;

    switch (mac->mac_unit) {
        case 0:
            /*--- External RMII Mode on GE0,Master and High speed ---*/
            AVMNET_INFO("set External RMII Mode on GE0\n");
            ath_reg_wr(ATHR_GMAC_ETH_CFG, ATHR_GMAC_ETH_CFG_RGMII_GE0);
            break;
        case 1:
            /*--- athr_swap_phy(); ---*/
            break;
    }

    if (is_ar934x()) {
        ath_reg_rmw_set(SWITCH_CLOCK_SPARE_ADDRESS, (1 << 6));      /*--- MDIO_CLK = 100MHz ---*/
    }

    mgmt_cfg_val = 0x6;
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_CFG, mgmt_cfg_val | (1 << 31));
    athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_MII_MGMT_CFG, mgmt_cfg_val);

}

/*--- atomic_t athr_rx_disable_count; ---*/

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int athr_gmac_napi_poll(struct napi_struct *napi, int budget) {

    struct athmac_context *mac = (struct athmac_context *)container_of(napi, struct athmac_context, napi);
    avmnet_module_t *this = mac->this_module;
    struct net_device *dev = mac->device;


    int work_done = 0, status = 0;
    int max_work  = budget;
    athr_gmac_rx_status_t  ret;

    ret = athr_gmac_recv_packets(dev, this, max_work, &work_done);

    /*--- AVMNET_INFO("[%s] device %s ret 0x%x\n", __func__, dev->name, ret); ---*/

    /*------------------------------------------------------------------------------------------*\
     * Fix ME: 2.6.31 kernel hits a corner case when budget == work done
     * It tries to queue another request assuming that more packets are ready
     * to be processed and if napi_complete is called by the driver it
     * results in panic. Might not work for 2.6.15
    \*------------------------------------------------------------------------------------------*/
    if (likely(ret == ATHR_GMAC_RX_STATUS_DONE) && (work_done < budget)) {
        napi_complete(&mac->napi);
        athr_gmac_intr_enable_recv(mac->BaseAddress);
    } else if (likely(ret == ATHR_GMAC_RX_STATUS_NOT_DONE)) {
        /*--- We have work left ---*/
        status = 1;
        napi_complete(&mac->napi);
        napi_reschedule(&mac->napi);
    } else if (ret == ATHR_GMAC_RX_STATUS_OOM) {
        printk(KERN_ERR "[%s] athr_gmac_recv_packets: ATHR_GMAC_RX_STATUS_OOM\n", __FUNCTION__);
        AVMNET_INFO("[%s] oom..?\n", __func__);
        /*--- Start timer, stop polling, but do not enable rx interrupts. ---*/
        napi_complete(&mac->napi);
        mod_timer(&mac->mac_oom_timer, jiffies + 5);
    } else if(work_done < budget) {
        printk("budget used up\n");
        status = 1;
    }

    return status;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void athr_gmac_setup_eth( struct net_device* device) {

    ether_setup(device);           /*--- setup eth-defaults ---*/

    device->hard_header_len += 48;

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int avmnet_set_macaddr(int device_nr, struct net_device *dev);
void athr_gmac_setup_eth_priv(avmnet_device_t *avm_dev) {

    avmnet_netdev_priv_t *priv;
    struct net_device *device = avm_dev->device;

    AVMNET_INFO("[%s] devicename %s module %s\n", __func__, device->name, avm_dev->mac_module->name);

    priv = netdev_priv( device );

    if ( priv )
        priv->avm_dev = avm_dev;
    else {
        AVMNET_ERR("[%s] ERROR: priv = %p  avm_dev = %p\n", __func__, priv, avm_dev);
        return;
    }

    if ( ! strncmp("eth0", avm_dev->device_name, sizeof("eth0")-1)) {
        if (avmnet_set_macaddr(0, device)) {
            AVMNET_ERR("[%s] ERROR: %s set mac-Address\n" , __func__, avm_dev->device_name);
            BUG_ON(1);
        }
    }
    if ( ! strncmp("eth1", avm_dev->device_name, sizeof("eth1")-1)) {
        if (avmnet_set_macaddr(1, device)) {
            AVMNET_ERR("[%s] ERROR: %s set mac-Address\n" , __func__, avm_dev->device_name);
            BUG_ON(1);
        }
    }
    if ( ! strncmp("eth2", avm_dev->device_name, sizeof("eth2")-1)) {
        if (avmnet_set_macaddr(2, device)) {
            AVMNET_ERR("[%s] ERROR: %s set mac-Address\n" , __func__, avm_dev->device_name);
            BUG_ON(1);
        }
    }
    if ( ! strncmp("eth3", avm_dev->device_name, sizeof("eth3")-1)) {
        if (avmnet_set_macaddr(3, device)) {
            AVMNET_ERR("[%s] ERROR: %s set mac-Address\n" , __func__, avm_dev->device_name);
            BUG_ON(1);
        }
    }
    if ( ! strncmp("plc", avm_dev->device_name, sizeof("plc")-1)) {
        if (avmnet_set_macaddr(2, device)) {
            AVMNET_ERR("[%s] ERROR: %s set mac-Address\n" , __func__, avm_dev->device_name);
            BUG_ON(1);
        }
    }
     
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_gmac_setup(avmnet_module_t *this) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;
    int i, result = -1;
    
    AVMNET_TRC("[%s] Setup on module %s called.\n", __func__, this->name);
    
    /*--- reset HW ---*/
    athr_gmac_reg_rmw_set(mac->BaseAddress, ATHR_GMAC_CFG1, 
            ATHR_GMAC_CFG1_SOFT_RST | ATHR_GMAC_CFG1_RX_RST | ATHR_GMAC_CFG1_TX_RST);


#if defined(ATH_FLAGS)
    if (mac_has_flag(mac,WAN_QOS_SOFT_CLASS))
        mac->mac_noacs = 4;
    else
#endif
        mac->mac_noacs = 1;

    /*--- mac->tx_ac_bt = 0; ---*/
    /*--- mac->tx_ac_drops = 0; ---*/

    if (athr_gmac_tx_alloc(mac)) goto tx_failed;
    if (athr_gmac_rx_alloc(mac)) goto rx_failed;

    athmac_gmac_setup_mii(mac);
    /*--- athr_init_hwaccels(mac); ---*/
    /*------------------------------------------------------------------------------------------*\
     * muss auf jeden Fall vor dem Setup der PHYs aufgerufen werden!!! Atheos-Phy latchen die Config 
     * beim Reset an den MII Pins, das Interface muss als am Prozessor schon initialisiert sein
    \*------------------------------------------------------------------------------------------*/
    athmac_gmac_setup_hw(mac);

    register_netdev( mac->device );

    netif_napi_add(mac->device , &mac->napi, athr_gmac_napi_poll, ATHR_GMAC_NAPI_WEIGHT);
    netif_carrier_off(mac->device);
    netif_stop_queue(mac->device);

    napi_enable(&mac->napi);

    /* set the mac addr */  /*--- nicht unbedingt Notwendig ---*/
#if 0
    {
        unsigned int w1 = 0, w2 = 0;
        addr_to_words(device->dev_addr, w1, w2);
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_GE_MAC_ADDR1, w1);
        athr_gmac_reg_wr(mac->BaseAddress, ATHR_GMAC_GE_MAC_ADDR2, w2);
    }
#endif

    if (this->initdata.mac.flags & AVMNET_CONFIG_FLAG_IRQ) {
        result = request_irq(mac->irq, athr_gmac_intr, IRQF_DISABLED, this->name, this);
        if(result < 0) {
            AVMNET_ERR("[%s] <ERROR: request irq %s - %d failed>\n", __func__, this->name, mac->irq);
            return -EINTR;
        }
    }

    for(i = 0; i < this->num_children; ++i){
        result = this->children[i]->setup(this->children[i]);
        if(result != 0){
            // handle error
        }
    }

    athr_gmac_rx_start(mac->BaseAddress);
    avmnet_cfg_register_module(this);

    athmac_setup_irq(this, 1);

    return 0;

rx_failed:
    athr_gmac_tx_free(mac);
tx_failed:
    return 1;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int gmac_open(struct net_device *device __attribute__ ((unused))) {
    AVMNET_TRC("[%s] %s\n", __func__, ((struct athmac_context *)netdev_priv(device))->this_module->name);
    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int gmac_stop(struct net_device *device __attribute__ ((unused))) {
    AVMNET_TRC("[%s] %s\n", __func__, ((struct athmac_context *)netdev_priv(device))->this_module->name);
    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void gmac_tx_timeout(struct net_device *device) {
    avmnet_module_t *this = ((struct athmac_context *)netdev_priv(device))->this_module;
    AVMNET_WARN("[%s] %s\n", __func__, this->name);
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int gmac_hard_start(struct sk_buff *skb, struct net_device *device __attribute__ ((unused))) {
    AVMNET_TRC("[%s] %s\n", __func__, ((struct athmac_context *)netdev_priv(device))->this_module->name);

    dev_kfree_skb_any(skb);
    return 0;
}

struct net_device_ops athmac_device_ops = {
        .ndo_open             = gmac_open,
        .ndo_stop             = gmac_stop,
        .ndo_tx_timeout       = gmac_tx_timeout,
        .ndo_start_xmit       = gmac_hard_start
};

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_gmac_init(avmnet_module_t *this) {

    struct net_device *device;
    struct athmac_context *mac;
    int i, result;

    AVMNET_INFOTRC("[%s] '%s'\n", __func__, this->name);

    device = alloc_netdev( sizeof(struct athmac_context), this->name, athr_gmac_setup_eth );
    if(!device) {
        AVMNET_ERR("[%s] Could not allocate net_device %s\n", __func__, this->name);
        return -1;
    }

    device->netdev_ops = &athmac_device_ops;
    this->priv = netdev_priv( device );

    mac = (struct athmac_context *)this->priv;

    mac->device = device;
    mac->this_module = this;
    mac->mac_unit = this->initdata.mac.mac_nr;

    if (this->initdata.mac.flags & AVMNET_CONFIG_FLAG_BASEADDR) {
        mac->BaseAddress = this->initdata.mac.base_addr;
    }

    if (this->initdata.mac.flags & AVMNET_CONFIG_FLAG_RESET) {
        mac->reset = this->initdata.mac.reset;
        /*--- Resourcen anmelden ---*/
        ath_gmac_gpio.start = mac->reset;
        ath_gmac_gpio.end = mac->reset;
        ath_gmac_gpio.name = this->name;
        if(request_resource(&gpio_resource, &ath_gmac_gpio)) {
            AVMNET_ERR("[%s] ERROR request resource gpio %d\n", __func__, mac->reset);
            kfree(mac);
            return -EIO;
        }
    }

    if (this->initdata.mac.flags & AVMNET_CONFIG_FLAG_IRQ) {
        mac->irq = this->initdata.mac.irq;
    }

    mac->num_tx_desc   = athr_tx_desc_num[mac->mac_unit];
    mac->num_rx_desc   = athr_rx_desc_num[mac->mac_unit];
    mac->reap_thresh   = athr_tx_desc_num[mac->mac_unit] / 2;
    mac->qstart_thresh = 5 * tx_max_desc_per_ds_pkt;
 
    spin_lock_init(&mac->spinlock);
    init_MUTEX(&mac->lock);

    init_timer(&mac->mac_oom_timer);
    mac->mac_oom_timer.data     = (unsigned long)this;
    mac->mac_oom_timer.function = (void *)athr_gmac_oom_timer;

    /*--- INIT_WORK(&mac->mac_tx_timeout, athr_gmac_tx_timeout_task); ---*/

    tasklet_init(&mac->txreaptq, athr_gmac_txreap_task, (unsigned long)this);

    for(i = 0; i < this->num_children; ++i){
        result = this->children[i]->init(this->children[i]);
        if(result < 0){
            // handle error
        }
    }

    return 0;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athmac_gmac_exit(avmnet_module_t *this) {

    struct athmac_context *mac = (struct athmac_context *)this->priv;
    struct net_device *device = mac->device;
    int i, result;
   
    AVMNET_TRC("[%s] Exit module %s called.\n", __func__, this->name);

    for(i = 0; i < this->num_children; ++i){
        result = this->children[i]->exit(this->children[i]);
        if(result != 0){
            // handle error
        }
    }

    athr_gmac_rx_stop(mac->BaseAddress);
    athr_gmac_tx_stop(mac->BaseAddress);

    athmac_setup_irq(this, 0);
    netif_napi_del(&mac->napi);
    unregister_netdev(device);

    athr_gmac_tx_free(mac);
    athr_gmac_rx_free(mac);

    free_irq(mac->irq, athr_gmac_intr);

    free_netdev(device);   /*--- kein device kein mac ! ---*/

    return 0;
}

#if 0
/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athr_gmac_open_plc(struct net_device *device) {
    int ret;
    avmnet_device_t *avmdev;
    struct ethtool_cmd cmd;
    struct ethtool_pauseparam pause;

    AVMNET_TRC("[%s]\n", __func__);

    avmdev = get_avmdev_by_name("plc");
    if(avmdev == NULL) {
        AVMNET_ERR("[%s] Error! Could not get avmdev for 'plc'!\n", __func__);
        return -1;
    }
    if(0 != avmdev->device->ethtool_ops->get_settings(avmdev->device, &cmd)) {
        AVMNET_ERR("[%s] Error! Could not get the link settings for the plc port!\n", __func__);
        return -1;
    }

    pause.cmd = ETHTOOL_SPAUSEPARAM;
    pause.autoneg = 1;
    pause.rx_pause = 0;
    pause.tx_pause = 0;
    if(0 != avmdev->device->ethtool_ops->set_pauseparam(avmdev->device, &pause)) {
        AVMNET_ERR("[%s] Error! Could not set the pause parameters for the plc port!\n", __func__);
        return -1;
    }

    /*--- ethtool_cmd_speed_set(&cmd, 100); ---*/
    /*--- cmd.duplex = DUPLEX_FULL; ---*/
    /*--- cmd.eth_tp_mdix = ETH_TP_MDI; ---*/
    cmd.autoneg = AUTONEG_ENABLE;
    cmd.advertising =   ADVERTISED_100baseT_Half
                      | ADVERTISED_100baseT_Full
                      | ADVERTISED_Autoneg
                      | ADVERTISED_TP;

    if(0 != avmdev->device->ethtool_ops->set_settings(avmdev->device, &cmd)) {
        AVMNET_ERR("[%s] Error! Could not set the settings for the plc port!\n", __func__);
        return -1;
    }

    ret = athr_gmac_open(device);

    /* Get the PLC out of reset */
    avm_gpio_out_bit(20, 1);

    return ret;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
int athr_gmac_stop_plc(struct net_device *device) {
    AVMNET_TRC("[%s]\n", __func__);

    /* Put the PLC into reset */
    avm_gpio_out_bit(20, 0);

    return athr_gmac_stop(device);
}
#endif
